#!/usr/bin/perl -wT

use strict;
use Time::Local;
use List::Util qw(max reduce);

# generate iterator over lines of a file
sub lines($$) {
	my ($name, $shortname) = @_;
	open my $file, "<$name" or die "cannot open $name: $!\n";
	my $line = <$file>;
	sub {
		return $line unless @_;
		return $shortname if $_[0] eq 'name';
		return undef unless $_[0] eq 'next';
		$line = <$file>;
		$line =~ s/;.*//s if $line;
		$line;
	}
}

# mapping of month name abbreviations (Jan=>0, Feb=>1, ...)
my %months = do {
	my $count = 0;
	map { ($_ => $count++) } qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
};

# determine timestamp of log line, assumes format 'Jan 1 00:00:00'
sub mytime($) {
	my ($m, $d, $H, $M, $S) = $_[0] =~ /(\w+)\s+(\d+)\s+(\d+):(\d+):(\d+)/;
	$m = $months{$m};
	timelocal($S, $M, $H, $d, $m, 2000);
}

# sort file iterators in ascending order of the next timestamp
sub sortfiles(\@) {
	@{$_[0]} = sort { mytime($a->()) <=> mytime($b->()) } @{$_[0]};
}

# simple FP utility
sub forall(&@) {
	my ($cond, @list) = @_;
	@list == grep { $cond->() } @list;
}

# find longest common path prefix
my ($prefix) = $ARGV[0] =~ m,^(.*/),;
while ($prefix ne '' && $prefix ne '/') {
	last if forall { /^\Q$prefix/ } @ARGV;
	$prefix =~ s,/[^/]*/$,/,;
}
my $prefixlen = length $prefix;
my %shortnames = do {
	if ($prefixlen > 1) {
		map { ($_ => '.../'.substr($_, $prefixlen)) } @ARGV;
	} else {
		map { ($_ => $_) } @ARGV;
	}
};

my %unique;
map {
	m,([^/]+)$,;
	if (exists $unique{$1}) {
		$unique{$1}++
	} else {
		$unique{$1} = 1
	}
} @ARGV;
%unique = map {
	m,([^/]+)$,;
	if ($unique{$1} == 1) {
		($_ => $1)
	}
} @ARGV;
map { $shortnames{$_} = $unique{$_} } keys %unique;

# generate line iterators for all files
my @files = map { lines($_, $shortnames{$_}) } @ARGV;

# determine maximum shortname length
my $maxlen = max map { length $_->('name') } @files;

# poll file iterators until all are exhausted
while (@files) {
	sortfiles @files;
	printf "%-${maxlen}s | %s", $files[0]->('name'), $files[0]->();
	# close file which is finished
	$files[0]->('next') or shift @files;
}
